(function () {

    'use strict';

    var trim = function (str) {
        return str.replace(/^\s+|\s+$/g,'');
    };

    var typeOf = function (arg) {
        if (arg === null) {
            return 'null';
        }
        if (Object.prototype.toString.call(arg) === '[object Array]') {
            return 'array';
        }
        return typeof arg;
    };

    var isPlainObject = function (obj) {
        return obj.constructor && ((!obj.hasOwnProperty('constructor') && obj.constructor.prototype === Object.prototype) || typeOf(obj.constructor) !== 'function');
    }

    var TypeNotMatchError = function (type, objKey) {
        var __error = new Error('JSON data type mismatched, value "' + objKey + '" is not a/an ' + type);
        this.name = 'TypeNotMatchError';
        this.message = __error.message;
        this.stack = __error.stack;
        return this;
    };

    var InvalidSchemaError = function (type) {
        var __error = new Error(type + ' is not a valid schema type(bool, number, string, array, object)');
        this.name = 'InvalidSchemaError';
        this.message = __error.message;
        this.stack = __error.stack;
        return this;
    }

    var MissingKeyError = function (key) {
        var __error = new Error('\'' + key + '\' is not in the provided JSON.');
        this.name = 'MissingKeyError';
        this.message = __error.message;
        this.stack = __error.stack;
        return this;
    }

    var jsonbase = {};

    jsonbase.DataTypes = {
        nul: null,
        bool:  true,
        array:  [],
        number: 1,
        string: "",
        object: {},
    };

    jsonbase.TypeNotMatchError = TypeNotMatchError;
    jsonbase.InvalidSchemaError = InvalidSchemaError;
    jsonbase.MissingKeyError = MissingKeyError;

    var toNull = function (value) {
        if ([null, 0, false, ''].indexOf(value) > -1) {
            return null;
        } else {
            throw new TypeNotMatchError('null', value);
        }
    };

    var toBoolean = function (value) {
        switch (typeOf(value)) {
            case 'null':
            case 'boolean':
                return Boolean(value);
            case 'number':
                if (value === 0 || value === 1) {
                    return Boolean(value);
                }
                break;
            case 'string':
                var __value = trim(value);
                if (['1', 'true'].indexOf(__value) > -1) {
                    return true;
                } else if (['0', 'false'].indexOf(__value) > -1) {
                    return false;
                }
                break;
        }
        throw new TypeNotMatchError('boolean', value);
    };

    var toNumber = function (value) {
        switch (typeOf(value)) {
            case 'null':
            case 'number':
            case 'boolean':
                return Number(value);
            case 'string':
                var __value = Number(value);
                if (!isNaN(__value)) {
                    return __value;
                }
                break;
        }
        throw new TypeNotMatchError('number', value);
    };

    var toString = function (value) {
        switch (typeOf(value)) {
            case 'null':
                return '';
            case 'number':
            case 'string':
            case 'boolean':
                return String(value);
        }
        throw new TypeNotMatchError('string', value);
    };

    // Will deep copy arrays and objects.
    jsonbase.parse = function (data, schema) {
        var result;
        var type = typeOf(schema);
        if (type === 'object' && !isPlainObject(schema)) {
            throw new InvalidSchemaError(type);
        };
        switch (type) {
            case 'null':
                result = toNull(data);
                break;
            case 'boolean':
                result = toBoolean(data);
                break;
            case 'number':
                result = toNumber(data);
                break;
            case 'string':
                result = toString(data);
                break;
            case 'array':
                if (typeOf(data) !== 'array') {
                    throw new TypeNotMatchError(type, data);
                }
                result = [];
                if (!schema.length) {
                    result = [].concat(data);
                } else {
                    for (var i = 0; i < data.length; i++) {
                        var childSchema = schema.length === 1 ? schema[0] : schema[i];
                        result.push(jsonbase.parse(data[i], childSchema));
                    }
                }
                break;
            case 'object':
                if (typeOf(data) !== 'object') {
                    throw new TypeNotMatchError(type, data);
                }
                result = {};
                for (var key in schema) {
                    if (!schema.hasOwnProperty(key)) {
                        continue;
                    }
                    if (typeOf(data[key]) === 'undefined') {
                        throw new MissingKeyError(key);
                    }
                    result[key] = jsonbase.parse(data[key], schema[key]);
                }
                break;
            default:
                throw new InvalidSchemaError(type);
                break;
        }
        return result;
    };

    if (typeof(require) !== 'undefined' &&
        typeof(module) !== 'undefined' &&
        typeof(exports) !== 'undefined') {
        module.exports = jsonbase;                          // Node.js
    } else if (typeof(define) !== 'undefined') {
        if(define.amd) {
            define(function () {
                return jsonbase;                            // AMD
            });
        } else if(define.cmd) {
            define(function (require, exports, module) {
                module.exports = jsonbase;                  // CMD
            });
        }
    } else if (typeof(window) !== 'undefined') {
        window['jsonbase'] = jsonbase;                      // Native
    }

})();