var expect = require('chai').expect;
var jsonbase = require('../index.js');
var DataTypes = jsonbase.DataTypes;
var TypeNotMatchError = jsonbase.TypeNotMatchError;
var InvalidSchemaError = jsonbase.InvalidSchemaError;
var MissingKeyError = jsonbase.MissingKeyError;

describe('Test jsonbase.parse with data type: null', function () {
    it('Should get nulls from null inputs: null => null', function () {
        expect(jsonbase.parse(null, DataTypes.nul)).to.be.equal(null);
    });

    it('Should get nulls from false: false => null', function () {
        expect(jsonbase.parse(false, DataTypes.nul)).to.be.equal(null);
    });

    it('Should get nulls from 0: 0 => null', function () {
        expect(jsonbase.parse(0, DataTypes.nul)).to.be.equal(null);
    });

    it('Should get nulls from empty string inputs: \'\' => null', function () {
        expect(jsonbase.parse('', DataTypes.nul)).to.be.equal(null);
    });

    it('Should get TypeNotMatchError from other inputs', function () {
        expect(jsonbase.parse.bind(null, 2, DataTypes.nul)).to.throw(TypeNotMatchError);
        expect(jsonbase.parse.bind(null, '2', DataTypes.nul)).to.throw(TypeNotMatchError);
        expect(jsonbase.parse.bind(null, [], DataTypes.nul)).to.throw(TypeNotMatchError);
        expect(jsonbase.parse.bind(null, {}, DataTypes.nul)).to.throw(TypeNotMatchError);
    });
});

describe('Test jsonbase.parse with data type: Boolean', function () {
    it('Should get booleans from null inputs: null => false', function () {
        expect(jsonbase.parse(null, DataTypes.bool)).to.be.equal(false);
    });

    it('Should get booleans from boolean inputs: true => true; false => false', function () {
        expect(jsonbase.parse(true, DataTypes.bool)).to.be.equal(true);
        expect(jsonbase.parse(false, DataTypes.bool)).to.be.equal(false);
    });

    it('Should get booleans from 0 or 1: 1 => true; 0 => false', function () {
        expect(jsonbase.parse(1, DataTypes.bool)).to.be.equal(true);
        expect(jsonbase.parse(0, DataTypes.bool)).to.be.equal(false);
    });

    it('Should get booleans from string inputs: \'true\' => true; \'false\' => false', function () {
        expect(jsonbase.parse('true', DataTypes.bool)).to.be.equal(true);
        expect(jsonbase.parse('false', DataTypes.bool)).to.be.equal(false);
    });

    it('Should get TypeNotMatchError from other inputs', function () {
        expect(jsonbase.parse.bind(null, 2, DataTypes.bool)).to.throw(TypeNotMatchError);
        expect(jsonbase.parse.bind(null, '2', DataTypes.bool)).to.throw(TypeNotMatchError);
        expect(jsonbase.parse.bind(null, [], DataTypes.bool)).to.throw(TypeNotMatchError);
        expect(jsonbase.parse.bind(null, {}, DataTypes.bool)).to.throw(TypeNotMatchError);
    });
});

describe('Test jsonbase.parse with data type: Number', function () {
    it('Should get numbers from null inputs: null => 0', function () {
        expect(jsonbase.parse(null, DataTypes.number)).to.be.equal(0);
    });

    it('Should get numbers from number inputs: 1 => 1', function () {
        expect(jsonbase.parse(1, DataTypes.number)).to.be.equal(1);
    });

    it('Should get numbers from \"number\" string inputs: \'255\' => 255', function () {
        expect(jsonbase.parse('255', DataTypes.number)).to.be.equal(255);
    });

    it('Should get numbers from boolean inputs: true => 1; false => 0', function () {
        expect(jsonbase.parse(true, DataTypes.number)).to.be.equal(1);
        expect(jsonbase.parse(false, DataTypes.number)).to.be.equal(0);
    });

    it('Should get TypeNotMatchError from other inputs', function () {
        expect(jsonbase.parse.bind(null, '1a', DataTypes.number)).to.throw(TypeNotMatchError);
        expect(jsonbase.parse.bind(null, [], DataTypes.number)).to.throw(TypeNotMatchError);
        expect(jsonbase.parse.bind(null, {}, DataTypes.number)).to.throw(TypeNotMatchError);
    });
});

describe('Test jsonbase.parse with data type: String', function () {
    it('Should get empty strings from null inputs: null => \'\'', function () {
        expect(jsonbase.parse(null, DataTypes.string)).to.be.equal('');
    });

    it('Should get strings from boolean inputs: true => \'true\'; false => \'false\'', function () {
        expect(jsonbase.parse(true, DataTypes.string)).to.be.equal('true');
        expect(jsonbase.parse(false, DataTypes.string)).to.be.equal('false');
    });

    it('Should get strings from number inputs: 1 => \'1\'', function () {
        expect(jsonbase.parse(1, DataTypes.string)).to.be.equal('1');
    });

    it('Should get strings from string inputs: \'abc\' => \'abc\'', function () {
        expect(jsonbase.parse('abc', DataTypes.string)).to.be.equal('abc');
    });

    it('Should get TypeNotMatchError from other inputs', function () {
        expect(jsonbase.parse.bind(null, [], DataTypes.string)).to.throw(TypeNotMatchError);
        expect(jsonbase.parse.bind(null, {}, DataTypes.string)).to.throw(TypeNotMatchError);
    });
});

describe('Test jsonbase.parse with data type: Array', function () {
    it('Should get same arrays from simple array inputs: [1, 2, 3, ...] => [1, 2, 3, ...]', function () {
        expect(jsonbase.parse([1, 2, 3, 4], DataTypes.array)).to.eql([1, 2, 3, 4]);
        expect(jsonbase.parse([1, 2, 3, 4], [DataTypes.number])).to.eql([1, 2, 3, 4]);
        expect(jsonbase.parse([true, false], [DataTypes.bool])).to.eql([true, false]);
        expect(jsonbase.parse(['a', 'b', 'c'], [DataTypes.string])).to.eql(['a', 'b', 'c']);
    });

    it('Should get same arrays from array inputs with mixed data type: [1, false, \'a\', ...] => [1, false, \'a\', ...]', function () {
        expect(jsonbase.parse([1, false, 'a'], [DataTypes.number, DataTypes.bool, DataTypes.string])).to.eql([1, false, 'a']);
    });

    it('Should get same arrays from nested array inputs: [[1, 2], [3, 4], ...] => [[1, 2], [3, 4], ...]', function () {
        expect(jsonbase.parse([[1, 2], [3, 4]], [[DataTypes.number]])).to.eql([[1, 2], [3, 4]]);
        expect(jsonbase.parse([[[1], [2]], [[3], [4]]], [[[DataTypes.number]]])).to.eql([[[1], [2]], [[3], [4]]]);
    });

    it('Should get TypeNotMatchError from other inputs', function () {
        expect(jsonbase.parse.bind(null, null, DataTypes.array)).to.throw(TypeNotMatchError);
        expect(jsonbase.parse.bind(null, true, DataTypes.array)).to.throw(TypeNotMatchError);
        expect(jsonbase.parse.bind(null, 1, DataTypes.array)).to.throw(TypeNotMatchError);
        expect(jsonbase.parse.bind(null, {}, DataTypes.array)).to.throw(TypeNotMatchError);
    });
});

describe('Test jsonbase.parse with data type: Object', function () {
    it('Should get empty objects if no schema object key specified: \
{ name: \'Mike\', age: 12, student: true } => {}', function () {
        expect(jsonbase.parse({ name: 'Mike', age: 12, student: true }, DataTypes.object)).to.eql({});
    });

    it('Should get objects if parts of object keys are specified in the schema: \
{ name: \'Mike\', age: 12, student: true } => { name: \'Mike\', student: true }', function () {
        expect(jsonbase.parse({ name: 'Mike', age: 12, student: true }, { name: DataTypes.string, student: DataTypes.bool })).to.eql({ name: 'Mike', student: true });
    });

    it('Should get objects from nested object inputs: { name: \'Mike\', hobbies: [ \'swim\' ], \
brother: { name: \'Joe\' } } => { name: \'Mike\', hobbies: [ \'swim\' ], brother: { name: \'Joe\' } }', function () {
        expect(jsonbase.parse({ name: 'Mike', hobbies: [ 'swim' ], brother: { name: 'Joe' } },
            { name: DataTypes.string, hobbies: [ DataTypes.string ], brother: { name: DataTypes.string } }))
            .to.eql({ name: 'Mike', hobbies: [ 'swim' ], brother: { name: 'Joe' } });
    });

    it('Should get MissingKeyError if some schema keys are not provided in the object', function () {
        expect(jsonbase.parse.bind(null, { name: 'Mike', age: 12, student: true },
            { name: DataTypes.string, gender: DataTypes.string })).to.throw(MissingKeyError);
    });
});

describe('Test jsonbase.parse with invalid schema type', function () {
    it('Should throw InvalidSchemaError error', function () {
        expect(jsonbase.parse.bind(null, true, new Date())).to.throw(InvalidSchemaError);
        expect(jsonbase.parse.bind(null, 1, function () {})).to.throw(InvalidSchemaError);
    });
});
